/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../../state/reducers";
import {FileWithData} from "../../../../types/exerciseEditor";
import {List, Icon} from "semantic-ui-react";
import {copyToClipboard} from "../../../../helpers/clipboard";
import {convertSizeToHumanReadableSize} from "../../../../helpers/convertersAndTransformers";
import {setExerciseEditorAssets} from "../../../../state/actions/exerciseEditorSite/subSets/editorExerciseDescriptionActions";
import {getCopyAssetUrlText} from "../../../../helpers/assetHelper";
import {assetPrefix} from '../../../../constants'
import Logger from '../../../../helpers/logger'
const fileSaver = require('file-saver');

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly assets: ReadonlyArray<FileWithData>
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    ...props,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


class AssetsPanelView extends React.Component<Props, any> {
  render(): JSX.Element {
    return (
      <div>

        <List>
          {
            this.props.assets.map((asset: FileWithData) => {
              return (
                <List.Item key={asset.id}>
                  <List.Icon size='large' verticalAlign='middle'>
                    <div className="clickable" onClick={() => {

                      if (asset.content === null) {
                        Logger.error('file content was not found')
                        return
                      }

                      // noinspection TsLint
                      let fileName = asset.displayName.trim()

                      if (!fileName) {
                        fileName = "invalid_filename.data"
                      }


                      const bytes = new Uint8Array(asset.content);
                      // tslint:disable-next-line
                      const blob = new Blob([bytes], {type: asset.mimeType});
                      fileSaver.saveAs(blob, fileName)
                    }}>
                      <Icon name="download" />
                    </div>
                  </List.Icon>
                  <List.Content>
                    <List.Header>
                      <div className="flexed">
                        <div className="mar-right-half">
                          {
                            asset.displayName
                          }
                        </div>

                        <div className="not-important-colored">
                          (
                          {convertSizeToHumanReadableSize(asset.content.length)}
                          )
                        </div>
                      </div>
                    </List.Header>
                    <List.Description>
                      <div className="flexed">
                        <div className="mar-right-half">
                          {
                            `${assetPrefix}${asset.id}`
                          }
                        </div>
                        <div className="clickable"
                             onClick={() => {
                               copyToClipboard(getCopyAssetUrlText(asset))
                             }}
                        >
                          <Icon name="copy"/>
                        </div>
                      </div>
                    </List.Description>
                  </List.Content>
                </List.Item>
              )
            })
          }
        </List>

      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(AssetsPanelView)
