/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {FilePreviewFromBackend} from "../../../../../types/exerciseEditor";
import {RootState} from "../../../../../state/reducers";
import {Icon, List} from "semantic-ui-react";
import {convertSizeToHumanReadableSize} from "../../../../../helpers/convertersAndTransformers";
import {copyToClipboard} from '../../../../../helpers/clipboard'
import {getCopyAssetUrlText} from '../../../../../helpers/assetHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string

  readonly files: ReadonlyArray<FilePreviewFromBackend>

  /**
   * to re use this component a generic get download file href func
   */
  readonly getDownloadFileHRef: (file: FilePreviewFromBackend) => string

}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    langId: rootState.i18nState.langId,
    ...props
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here

                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class changeTestFilesListView extends React.Component<Props, any> {
  render(): JSX.Element {


    return (
      <div>
        <List>
          {
            this.props.files.map((file: FilePreviewFromBackend) => {


              return (
                <List.Item key={file.id}>
                    <List.Icon size='large' verticalAlign='middle'>

                      <a target="_blank" download
                         href={this.props.getDownloadFileHRef(file)}>
                        <Icon name="download"/>
                      </a>

                  </List.Icon>
                  <List.Content>
                    <List.Header>

                      <div className="flexed">
                        <div className="mar-right-half">

                          <span>
                            {
                              file.originalName
                            }
                          </span>
                        </div>

                        <div className="clickable"
                             onClick={() => {
                               copyToClipboard(file.originalName)
                             }}
                        >
                          <Icon name="copy"/>
                        </div>

                        <div className="not-important-colored">
                          (
                          {convertSizeToHumanReadableSize(file.sizeInBytes)}
                          )
                        </div>
                      </div>

                    </List.Header>
                    <List.Description>
                      <div className="flexed">
                        <div className="not-important-colored mar-right-half">
                          {
                            file.mimeType
                          }
                        </div>

                      </div>
                    </List.Description>
                  </List.Content>
                </List.Item>
              )
            })
          }
        </List>
      </div>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(changeTestFilesListView)
