/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {
  removeMultipleMemberAndRefreshAsync, setIsAddUsersDialogDisplayed,
} from "../../../state/actions/singleGroupSite/singleGroupSiteActions";
import {Icon} from "semantic-ui-react";
import {SimpleVDivider} from "../../helpers/simpleVDivider";
import CommonHeaderBarContentWrapper from '../../commonHeaderBarContentWrapper'


import {askDialog} from "../../../helpers/dialogHelper";
import {getI18n} from "../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    isLoading: rootState.singleGroupState.isLoading,
    groupId: rootState.singleGroupState.selectedGroupId,
    defaultUserGroupId: rootState.globalSystemSettingsState.systemSettings.defaultUserGroupId,
    group: rootState.singleGroupState.group,
    members: rootState.singleGroupState.members,
    isSomeDialogDisplayed: rootState.singleGroupState.isAddUsersDialogDisplayed,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
  //imported reducer funcs here
  setIsAddUsersDialogDisplayed,
  removeMultipleMemberAndRefreshAsync
}, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

class headerBarContent extends React.Component<Props, any> {
  render(): JSX.Element {

    const isSomeUserSelected = this.props.members.some(p => p.uiIsSelected)

    const currentUserGroupRole = this.props.group === null ? null : this.props.group.groupRole

    const isDefaultUserGroup = this.props.groupId === this.props.defaultUserGroupId

    const leftArea = (
      <div>

        {
          (currentUserGroupRole && currentUserGroupRole.canAddUserToGroup) &&
          <div className="clickable"
               onClick={() => this.props.setIsAddUsersDialogDisplayed(true)}
          >
            <Icon name="plus"/>
            <span>
            {
              getI18n(this.props.langId,'Add member')
            }
          </span>
          </div>
        }

        {
          (currentUserGroupRole && currentUserGroupRole.canAddUserToGroup) &&
          <SimpleVDivider/>
        }

        {
          (currentUserGroupRole && currentUserGroupRole.canRemoveMemberFromGroup)&&
          <div className={isSomeUserSelected && isDefaultUserGroup === false  ? 'clickable' : 'div-disabled'}
               onClick={async () => {

                 if (isDefaultUserGroup) return

                 const userIds =
                   this.props.members
                     .filter(p => p.uiIsSelected)
                     .map(p => p.id)

                 // noinspection TsLint
                 let shouldRelease = true

                 if (userIds.length > 0) {

                   shouldRelease = await askDialog(getI18n(this.props.langId,'Release multiple users from group'),
                     `${getI18n(this.props.langId,'Are you sure you want to release multiple users from this group?')} (${userIds.length} users)`, this.props.langId
                   )
                 }

                 if (shouldRelease) {
                   this.props.removeMultipleMemberAndRefreshAsync(userIds, this.props.groupId)
                 }

               }}
          >
            <Icon name="remove"/>
            <span>
            {
              getI18n(this.props.langId,'Release selected member')
            }
               </span>
          </div>
        }


      </div>
    )


    return (
      <CommonHeaderBarContentWrapper leftArea={leftArea} disabled={this.props.isLoading || this.props.isSomeDialogDisplayed}/>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(headerBarContent)
