/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import SinglePanelSiteWrapper from '../../singlePanelSiteWrapper'
import MaterialSearchInput from '../../material/materialSearchInput'
import MemberListView from './memberListView'
import * as constants from '../../../constants'
import {Link} from "react-router-dom";
import AddUserDialog from './dialogs/addUserDialog/addUserDialog'
import {RouteComponentProps, withRouter} from "react-router";
import {
  loadSingleGroupSite,
  setSearchText,
  setSelectedGroupId
} from "../../../state/actions/singleGroupSite/singleGroupSiteActions";
import {parseId} from "../../../helpers/convertersAndTransformers";
import Spinner from "../../helpers/spinner";
import {getGroupsAsync} from "../../../state/actions/groupsSite/groupsCrudActions";
import {BackArrow} from "../../helpers/backArrow";
import {errorDialog} from '../../../helpers/dialogHelper'
import {getI18n} from '../../../../i18n/i18nRoot'
import {InputOnChangeEvent} from '../../../types/reactEvents'
import Logger from '../../../helpers/logger'
import {ErrorHelper} from '../../../helpers/errorHelper'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test
    //we cannot do this another way because there is no backend func to give us the group + members + roles...
    groups: rootState.singleGroupState.group,
    groupId: rootState.singleGroupState.selectedGroupId,
    isLoading: rootState.singleGroupState.isLoading,
    isAddUsersDialogDisplayed: rootState.singleGroupState.isAddUsersDialogDisplayed,
    groupRoles: rootState.singleGroupState.groupRoles,
    currentUserId: rootState.userDataSettingsSate.userData.id,
    group: rootState.singleGroupState.group,
    langId: rootState.i18nState.langId,
    pagination: rootState.singleGroupState.pagination,
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        loadSingleGroupSite,
                                                                        getGroupsAsync,
                                                                        setSelectedGroupId,
                                                                        setSearchText,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

interface MatchedProps {
  readonly id: string
}

class SingleGroupSite extends React.Component<Props & RouteComponentProps<MatchedProps>, any> {

  async componentWillMount(): Promise<void> {

    const groupId = parseId(this.props.match.params.id)

    try {
      if (groupId !== null) {
        await this.props.loadSingleGroupSite(groupId)
      } else {
        errorDialog(getI18n(this.props.langId, "Error"), getI18n(this.props.langId, "Could not load site data"),this.props.langId)
        Logger.error('Could not load site data because groupId was null')
      }
    } catch(err) {
      ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error', 'Could not load site data')
    }

  }

  render(): JSX.Element {


    return (
      <div>
        <SinglePanelSiteWrapper>

          {
            this.props.isLoading &&
            <Spinner/>
          }

          {
            !this.props.isAddUsersDialogDisplayed &&
            <div className="view-padding">
              <div className="view-head-line">
                <div className="view-caption">

                  <div className="mar-right back-arrow">
                    <Link to={constants.groupsLinkPath}>
                      <BackArrow/>
                    </Link>
                  </div>

                  <h1 className="clear-margin">
                    {
                      this.props.group &&
                      this.props.group.displayName
                    }
                  </h1>

                </div>
                <div className="view-options">
                  <MaterialSearchInput
                    value={this.props.pagination.searchText}
                    onChange={(e: InputOnChangeEvent) => {
                      this.props.setSearchText(e.currentTarget.value)
                    }}
                  />
                </div>
              </div>

              <div className="view-content">
                <MemberListView/>
              </div>
            </div>
          }

          {
            this.props.isAddUsersDialogDisplayed &&
            <AddUserDialog/>
          }


        </SinglePanelSiteWrapper>
      </div>
    )
  }
}

export default withRouter(connect(mapStateToProps, mapDispatchToProps)(SingleGroupSite))
