/**
 * Created by janis dähne.
 */
import {delay} from "../../../../debug";
import {ActionType} from "../../reducers/doExerciseSite/doExerciseSiteActionTypes";
import {GET_doExerciseAction} from "../../reducers/doExerciseSite/crud/getDoExerciseReducer";
import {
  getAllTestResults,
  getDoExercise,
  getDoExerciseSolution,
  getPossibleAllUserFiles,
  getSubmitTestResults,
  resetSolution,
  saveSolutionVersion,
  setLockSolutions,
  setShouldNotCount
} from "../../../communicationLayer/doExerciseLayer";
import {GET_doExercise_solutionVersionAction} from "../../reducers/doExerciseSite/crud/getSolutionReducer";
import {GET_allTestResultsAction} from "../../reducers/doExerciseSite/crud/getAllTestResultsReducer";
import {UPDATE_doExerciseSolutionAction} from "../../reducers/doExerciseSite/crud/updateDoEditorSolutionReducer";
import {SolutionDoExerciseFullBase} from "../../../types/exerciseSolution";
import {RUN_testAction} from "../../reducers/doExerciseSite/crud/runTestReducer";
import {compileSingleFile, justRunProgramTest, runAfterTest, runTest} from "../../../communicationLayer/testingLayer";
import {
  JustRunProgramCommandForBackend,
  RunCompileSingleFileCommandForBackend,
  RunTestCommandForBackend
} from "../../../types/testsAndTestProtocol";
import {RUN_customTestAction} from "../../reducers/doExerciseSite/crud/runCustomTestReducer";
import {RUN_compileSingleFileAction} from "../../reducers/doExerciseSite/crud/runCompileSingleFileReducer";
import {AwaitActions, AwaitActionsWithResult} from "../types";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {setTestsAreRunningAndResetResult} from "./sub/testActions";
import {setCustomTestIsRunning} from "./sub/customTestActions";
import {
  RunnableAfterCustomTestFrontendExclusive,
  RunnableCustomTestFrontendExclusive
} from "../../../types/customTests";
import {resetDoExerciseTestResults, setSelectedConsoleViewTest} from "./doExerciseSiteActions";
import {setHasFileChangedMapDoEditor, setHasUnsavedChanges} from "./editorActions/doEditorActions";
import {KnownInternalTestTypes, TestTypeFullBase} from "../../../types/testTypes";
import {GET_submitTestResultsAction} from "../../reducers/doExerciseSite/crud/getSubmitTestResults";
import {errorDialog, warningDialog} from "../../../helpers/dialogHelper";
import {setAfterSubmitTestsAreRunningAndResetResult} from "./sub/afterSubmitTestActions";
import {RUN_afterNormalTestAction} from "../../reducers/doExerciseSite/crud/afterSolution/runAfterNormalTest";
import {setAfterNormalTestsAreRunningAndResetResult} from "./sub/afterNormalTestActions";
import {AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {RUN_afterSubmitTestAction} from "../../reducers/doExerciseSite/crud/afterSolution/runAfterSubmitTest";
import {SET_shouldNotCountParticipationAction} from "../../reducers/doExerciseSite/crud/setShouldNotCountParticipationReducer";
import {JUST_RUN_programAction} from "../../reducers/doExerciseSite/crud/justRunProgramReducer";
import {getI18n, KnownLangs} from "../../../../i18n/i18nRoot";
import globalState from '../../state'
import {setAfterCustomTestsAreRunningAndResetResult} from './sub/afterCustomTestActions'
import {GET_allAfterTestResultsAction} from '../../reducers/doExerciseSite/crud/afterSolution/getAllAfterTestResultsReducer'
import {GET_afterSubmitTestResultsAction} from '../../reducers/doExerciseSite/crud/afterSolution/getAfterSubmitTestResultsReducer'
import {
  getAllAfterTestResults,
  getSubmitTestsAndResults
} from '../../../communicationLayer/doExerciseAfterSolutionLayer'
import {RUN_afterCustomTestAction} from '../../reducers/doExerciseSite/crud/afterSolution/runAfterCustomTest'
import {RESET_userSolutionAction} from '../../reducers/doExerciseSite/crud/resetUserSolution'
import {UPDATE_lockSolutionsFlagAction} from '../../reducers/doExerciseSite/crud/setLockSolutionsFlagReducer'
import {ErrorHelper} from '../../../helpers/errorHelper'
import {GET_allUserAssetPreviewsAction} from '../../reducers/doExerciseSite/crud/assets/getAllUserAssetPreviewsReducer'

export function getDoExerciseAsync(releaseCode: string): GET_doExerciseAction {
  return {
    type: ActionType.GET_doExercise,
    payload: delay(getDoExercise(releaseCode))
  }
}


export function getDoExerciseSolutionAsync(releaseCode: string, pLangId: number): GET_doExercise_solutionVersionAction {
  return {
    type: ActionType.GET_doExercise_solutionVersion,
    payload: delay(getDoExerciseSolution(releaseCode, pLangId))
  }
}


export function getDoExerciseAllTestResults(releaseCode: string, pLangId: number): GET_allTestResultsAction {
  return {
    type: ActionType.GET_allTestResults,
    payload: delay(getAllTestResults(releaseCode, pLangId))
  }
}

export function getAllDoExerciseAfterTestResults(releaseCode: string, pLangId: number): GET_allAfterTestResultsAction {
  return {
    type: ActionType.GET_allAfterTestResults,
    payload: delay(getAllAfterTestResults(releaseCode, pLangId))
  }
}


export function getDoExerciseSubmitTestResults(releaseCode: string, pLangId: number): GET_submitTestResultsAction {
  return {
    type: ActionType.GET_submitTestResults,
    payload: delay(getSubmitTestResults(releaseCode, pLangId))
  }
}

export function getDoExerciseAfterSubmitTestResults(releaseCode: string, pLangId: number): GET_afterSubmitTestResultsAction {
  return {
    type: ActionType.GET_afterSubmitTestResults,
    payload: delay(getSubmitTestsAndResults(releaseCode, pLangId))
  }
}

/**
 * @ignore obsolete, currently not used/supported
 * @param releaseCode
 * @param shouldLockSolutions
 */
export function updateDoExerciseLockSolutionsFlagAsync(releaseCode: string, shouldLockSolutions: boolean): UPDATE_lockSolutionsFlagAction {
  return {
    type: ActionType.UPDATE_lockSolutionsFlag,
    payload: delay(setLockSolutions(releaseCode, shouldLockSolutions))
  }
}


/**
 * this also reset has unsaved changes
 * @param {string} releaseCode
 * @param shouldClearAllTestResults
 * @param {SolutionDoExerciseFullBase} solutionVersion
 * @param langId
 * @returns {UPDATE_doExerciseSolutionAction}
 */
export function saveDoExerciseSolution(releaseCode: string,  shouldClearAllTestResults: boolean, solutionVersion: SolutionDoExerciseFullBase, langId: KnownLangs): AwaitActionsWithResult<boolean> {
  return async (dispatch) => {

    try {
      await dispatch(_saveDoExerciseSolution(releaseCode, shouldClearAllTestResults, solutionVersion, langId))
    } catch(err) {

      ErrorHelper.makeServerErrorDialog(err, langId, 'Error', 'Could not save files')
      return false
    }

    if (shouldClearAllTestResults) {
      dispatch(resetDoExerciseTestResults())
    }

    dispatch(setHasUnsavedChanges(false))
    dispatch(setHasFileChangedMapDoEditor({}))


    return true
  }
}

export function _saveDoExerciseSolution(releaseCode: string, shouldClearAllTestResults: boolean, solutionVersion: SolutionDoExerciseFullBase, langId: KnownLangs): UPDATE_doExerciseSolutionAction {
  return {
    type: ActionType.UPDATE_doExerciseSolution,
    payload: delay(saveSolutionVersion(releaseCode, shouldClearAllTestResults, solutionVersion)),
    meta: {
      langId
    }
  }
}


/**
 * run the given tests, sets the test states to running and selects the last or the first failed test
 * @param {RunTestCommandForBackend} runNormalTestCommand
 * @returns {AwaitActionsWithResult<boolean>} true: all tests passed, false: at least 1 did not compile or not passed
 */
export function runDoExerciseTest(runNormalTestCommand: RunTestCommandForBackend): AwaitActionsWithResult<boolean> {
  return async (dispatch, getState) => {

    //if we run the same test twice and the output won't change then this looks like nothing happened...
    //set console to blank so that the user sees that something happened
    dispatch(setSelectedConsoleViewTest(null))

    const runningTests = runNormalTestCommand.testIds
      .map<RunnableExerciseTestFrontendExclusive>(testId =>
        //don't use allTests because saveDoExerciseSolution could have reset the test (results) and allTest would be an old array
        getState().doExerciseSiteState.exercise.tests
          .find(test =>
            test.id === testId
          )
      )

    //set test as running to show animation
    dispatch(setTestsAreRunningAndResetResult(runningTests, getState().doExerciseSiteState.exercise.tests))

    await dispatch(_runDoExerciseTest(runNormalTestCommand))

    const ranTests = runNormalTestCommand.testIds
      .map<RunnableExerciseTestFrontendExclusive>(testId =>
        getState().doExerciseSiteState.exercise.tests
          .find(test =>
            test.id === testId
          )
      )

    if (ranTests.length === 0) {
      warningDialog(
        getI18n(globalState.getState().i18nState.langId,'Warning'),
        getI18n(globalState.getState().i18nState.langId,'Probably no test has run'), globalState.getState().i18nState.langId
      )
      return
    }

    const firstFailedTest = ranTests.find(p => !p || !p.result || !p.result.hasCompiled || !p.result.passed)

    if (firstFailedTest) {
      //select the first failed test
      dispatch(setSelectedConsoleViewTest(firstFailedTest))
    } else {
      //select the last test
      dispatch(setSelectedConsoleViewTest(ranTests[ranTests.length - 1]))
    }

    return !!firstFailedTest
  }
}

export function _runDoExerciseTest(runNormalTestCommand: RunTestCommandForBackend): RUN_testAction {
  return {
    type: ActionType.RUN_test,
    payload: delay(runTest(runNormalTestCommand))
  }
}

/**
 *
 * @param {RunTestCommandForBackend} runNormalTestCommand
 * @param langId
 * @returns {AwaitActionsWithResult<boolean>} true: all tests passed, false: at least 1 did not compile or not passed
 */
export function runCustomDoExerciseTest(runNormalTestCommand: RunTestCommandForBackend, langId: KnownLangs): AwaitActionsWithResult<boolean> {
  return async (dispatch, getState) => {

    //save changes if code changed
    if (getState().doExerciseSiteState.doEditorState.hasUnsavedChanges) {
      //first save changes
      //normally this should be func arguments but would the component re render on every code change??
      //reset test results because editor has unsaved changes
     const hasSaved = await dispatch(
        saveDoExerciseSolution(getState().doExerciseSiteState.exercisePreviewFrontendTemp.generatedCode, true, getState().doExerciseSiteState.solution, langId))

      if (!hasSaved) {
       return false
      }
    }

    //if we run the same test twice and the output won't change then this looks like nothing happened...
    //set console to blank so that the user sees that something happened
    dispatch(setSelectedConsoleViewTest(null))

    const runningTests = runNormalTestCommand.testIds
      .map<RunnableCustomTestFrontendExclusive>(testId =>
        //don't use allTests because saveDoExerciseSolution could have reset the test (results) and allTest would be an old array
        getState().doExerciseSiteState.exercise.customTests
          .find(test =>
            test.id === testId
          )
      )

    //set test as running to show animation
    dispatch(setCustomTestIsRunning(runningTests, getState().doExerciseSiteState.exercise.customTests))
    await dispatch(_runCustomDoExerciseTest(runNormalTestCommand))

    const updatedTest = getState().doExerciseSiteState.exercise.customTests.find(
      p => p.id === runNormalTestCommand.testIds[runNormalTestCommand.testIds.length - 1])

    if (!updatedTest) {
      warningDialog(
        getI18n(globalState.getState().i18nState.langId,'Warning'),
        getI18n(globalState.getState().i18nState.langId,'Probably no test has run'), globalState.getState().i18nState.langId
      )
      return
    }

    const ranTests = runNormalTestCommand.testIds
      .map<RunnableCustomTestFrontendExclusive>(testId =>
        getState().doExerciseSiteState.exercise.customTests
          .find(test =>
            test.id === testId
          )
      )

    const firstFailedTest = ranTests.find(p => !p || !p.result || !p.result.hasCompiled || !p.result.passed)

    if (firstFailedTest) {
      //select the first failed test
      dispatch(setSelectedConsoleViewTest(firstFailedTest))
    } else {
      //select the last test
      dispatch(setSelectedConsoleViewTest(updatedTest))
    }

    return !!firstFailedTest
  }
}

export function _runCustomDoExerciseTest(runNormalTestCommand: RunTestCommandForBackend): RUN_customTestAction {
  return {
    type: ActionType.RUN_customTest,
    payload: delay(runTest(runNormalTestCommand))
  }
}

export function runCompileSingleFileDoExercise(runCompileSingleFileCommand: RunCompileSingleFileCommandForBackend, testTypes: ReadonlyArray<TestTypeFullBase>): AwaitActions {
  return async (dispatch, getState) => {

    const compileTestType = testTypes.find(p => p.internalName === KnownInternalTestTypes.compileTest)

    if (!compileTestType) {

      errorDialog(getI18n(globalState.getState().i18nState.langId,'Error'),
        getI18n(globalState.getState().i18nState.langId,'Unknown test type'),globalState.getState().i18nState.langId)

      return
    }

    await dispatch(_runCompileSingleFileDoExercise(runCompileSingleFileCommand))

    const compileResult = getState().doExerciseSiteState.lastCompileSingleFileResult

    const tempTest: RunnableExerciseTestFrontendExclusive = {
      id: -1,
      displayName: 'Compile Single File',
      result: compileResult,
      content: '',
      uiIsRunning: false,
      files: [],
      displayIndex: -1,
      weight: 0,
      isSubmitTest: false,
      uiIsAfterTest: false,
      testSettings: {
        id: -1,
        maxDiskSpaceInKb: -1,
        timeoutInMs: -1,
        memoryLimitInKb: -1,
        compileTimeoutInMs: -1,
      },
      testTypeId: compileTestType.id
    }

    dispatch(setSelectedConsoleViewTest(tempTest))

  }
}

export function _runCompileSingleFileDoExercise(runCompileSingleFileCommand: RunCompileSingleFileCommandForBackend): RUN_compileSingleFileAction {
  return {
    type: ActionType.RUN_compileSingleFile,
    payload: delay(compileSingleFile(runCompileSingleFileCommand))
  }
}

export function justRunProgramDoExerciseAsync(justRunProgramCommand: JustRunProgramCommandForBackend, testTypes: ReadonlyArray<TestTypeFullBase>): AwaitActions {
  return async (dispatch, getState) => {

    // const justRunTestType = testTypes.find(p => p.internalName === KnownInternalTestTypes.justRunTest)
    //
    // if (!justRunTestType) {
    //
    //   errorDialog(getI18n(globalState.getState().i18nState.langId,'Error'),
    //     getI18n(globalState.getState().i18nState.langId,'Unknown test type'))
    //
    //   return
    // }

    await dispatch(_justRunProgramDoExercise(justRunProgramCommand))

    const testResult = getState().doExerciseSiteState.lastJustRunProgramResult

    const tempTest: RunnableExerciseTestFrontendExclusive = {
      id: -1,
      displayName: 'Just Run Program',
      result: testResult,
      content: '',
      uiIsRunning: false,
      files: [],
      displayIndex: -1,
      weight: 0,
      isSubmitTest: false,
      uiIsAfterTest: false,
      testSettings: {
        id: -1,
        maxDiskSpaceInKb: -1,
        timeoutInMs: -1,
        memoryLimitInKb: -1,
        compileTimeoutInMs: -1,
      },
      testTypeId: -1
    }

    dispatch(setSelectedConsoleViewTest(tempTest))

  }
}

export function _justRunProgramDoExercise(justRunProgramCommand: JustRunProgramCommandForBackend): JUST_RUN_programAction {
  return {
    type: ActionType.JUST_RUN_program,
    payload: delay(justRunProgramTest(justRunProgramCommand))
  }
}


//--- after normal tests, after submit tests

/**
 * run the given tests, sets the test states to running and selects the last or the first failed test
 * @returns {AwaitActionsWithResult<boolean>} true: all tests passed, false: at least 1 did not compile or not passed
 */
export function runAfterNormalTest(runTestWithSolution: RunTestCommandForBackend): AwaitActionsWithResult<boolean> {
  return async (dispatch, getState) => {
    //if we run the same test twice and the output won't change then this looks like nothing happened...
    //set console to blank so that the user sees that something happened
    dispatch(setSelectedConsoleViewTest(null))


    const runningTests = runTestWithSolution.testIds
      .map<RunnableExerciseTestFrontendExclusive>(testId =>
        //don't use allTests because saveDoExerciseSolution could have reset the test (results) and allTest would be an old array
        getState().doExerciseSiteState.afterNormalTests
          .find(test =>
            test.id === testId
          )
      )

    //set test as running to show animation
    dispatch(
      setAfterNormalTestsAreRunningAndResetResult(runningTests, getState().doExerciseSiteState.afterNormalTests))

    //run the test
    await dispatch(_runAfterNormalTest(runTestWithSolution))


    const ranTests = runTestWithSolution.testIds
      .map<RunnableExerciseTestFrontendExclusive>(testId =>
        getState().doExerciseSiteState.afterNormalTests
          .find(test =>
            test.id === testId
          )
      )

    if (ranTests.length === 0) {

      warningDialog(getI18n(globalState.getState().i18nState.langId,'Warning'),
        getI18n(globalState.getState().i18nState.langId,'Probably no test has run'), globalState.getState().i18nState.langId
      )
      return
    }

    const firstFailedTest = ranTests.find(p => !p || !p.result || !p.result.hasCompiled || !p.result.passed)

    if (firstFailedTest) {
      //select the first failed test
      dispatch(setSelectedConsoleViewTest(firstFailedTest))
    } else {
      //select the last test
      dispatch(setSelectedConsoleViewTest(ranTests[ranTests.length - 1]))
    }

    return !!firstFailedTest
  }
}

export function _runAfterNormalTest(runTestWithSolution: RunTestCommandForBackend): RUN_afterNormalTestAction {
  return {
    type: ActionType.RUN_afterNormalTest,
    payload: delay(runAfterTest(runTestWithSolution))
  }
}


/**
 * run the given tests, sets the test states to running and selects the last or the first failed test
 * @returns {AwaitActionsWithResult<boolean>} true: all tests passed, false: at least 1 did not compile or not passed
 */
export function runAfterSubmitTest(runTestWithSolution: RunTestCommandForBackend): AwaitActionsWithResult<boolean> {
  return async (dispatch, getState) => {
    //if we run the same test twice and the output won't change then this looks like nothing happened...
    //set console to blank so that the user sees that something happened
    dispatch(setSelectedConsoleViewTest(null))


    const runningTests = runTestWithSolution.testIds
      .map<AssessedSubmitTestFrontendExclusive>(testId =>
        //don't use allTests because saveDoExerciseSolution could have reset the test (results) and allTest would be an old array
        getState().doExerciseSiteState.afterSubmitTests
          .find(test =>
            test.id === testId
          )
      )

    //set test as running to show animation
    dispatch(
      setAfterSubmitTestsAreRunningAndResetResult(runningTests, getState().doExerciseSiteState.afterSubmitTests))

    //run the test
    await dispatch(_runAfterSubmitTest(runTestWithSolution))


    const ranTests = runTestWithSolution.testIds
      .map<AssessedSubmitTestFrontendExclusive>(testId =>
        getState().doExerciseSiteState.afterSubmitTests
          .find(test =>
            test.id === testId
          )
      )

    if (ranTests.length === 0) {

      warningDialog(getI18n(globalState.getState().i18nState.langId,'Warning'),
        getI18n(globalState.getState().i18nState.langId,'Probably no test has run'), globalState.getState().i18nState.langId
      )
      return
    }

    const firstFailedTest = ranTests.find(p => !p || !p.result || !p.result.hasCompiled || !p.result.passed)

    if (firstFailedTest) {
      //select the first failed test
      dispatch(setSelectedConsoleViewTest(firstFailedTest))
    } else {
      //select the last test
      dispatch(setSelectedConsoleViewTest(ranTests[ranTests.length - 1]))
    }

    return !!firstFailedTest
  }
}

export function _runAfterSubmitTest(runTestWithSolution: RunTestCommandForBackend): RUN_afterSubmitTestAction {
  return {
    type: ActionType.RUN_afterSubmitTest,
    payload: delay(runAfterTest(runTestWithSolution))
  }
}

/**
 * run the given tests, sets the test states to running and selects the last or the first failed test
 * @returns {AwaitActionsWithResult<boolean>} true: all tests passed, false: at least 1 did not compile or not passed
 */
export function runAfterCustomTest(runNormalTestCommand: RunTestCommandForBackend): AwaitActionsWithResult<boolean> {
  return async (dispatch, getState) => {
    //if we run the same test twice and the output won't change then this looks like nothing happened...
    //set console to blank so that the user sees that something happened
    dispatch(setSelectedConsoleViewTest(null))


    const runningTests = runNormalTestCommand.testIds
      .map<RunnableCustomTestFrontendExclusive>(testId =>
                                                  //don't use allTests because saveDoExerciseSolution could have reset the test (results) and allTest would be an old array
                                                  getState().doExerciseSiteState.exercise.customTests
                                                    .find(test =>
                                                            test.id === testId
                                                    )
      )

    //set test as running to show animation
    dispatch(
      setAfterCustomTestsAreRunningAndResetResult(runningTests, getState().doExerciseSiteState.afterCustomTests))

    //run the test
    await dispatch(_runAfterCustomTest(runNormalTestCommand))


    const updatedTest = getState().doExerciseSiteState.afterCustomTests.find(
      p => p.id === runNormalTestCommand.testIds[runNormalTestCommand.testIds.length - 1])

    if (!updatedTest) {
      warningDialog(
        getI18n(globalState.getState().i18nState.langId,'Warning'),
        getI18n(globalState.getState().i18nState.langId,'Probably no test has run'), globalState.getState().i18nState.langId
      )
      return
    }

    const ranTests = runNormalTestCommand.testIds
      .map<RunnableAfterCustomTestFrontendExclusive>(testId =>
                                                  getState().doExerciseSiteState.afterCustomTests
                                                    .find(test =>
                                                            test.id === testId
                                                    )
      )


    const firstFailedTest = ranTests.find(p => !p || !p.result || !p.result.hasCompiled || !p.result.passed)

    if (firstFailedTest) {

      const customTestCopy = getState().doExerciseSiteState.exercise.customTests.find(p => p.id === firstFailedTest.id)
      //select the first failed test
      dispatch(setSelectedConsoleViewTest({...customTestCopy, result: firstFailedTest.result}))
    } else {

      const customTestCopy = getState().doExerciseSiteState.exercise.customTests.find(p => p.id === ranTests[ranTests.length - 1].id)
      //select the last test
      dispatch(setSelectedConsoleViewTest({...customTestCopy, result: ranTests[ranTests.length - 1].result}))
    }

    return !!firstFailedTest
  }
}

export function _runAfterCustomTest(runTestWithSolution: RunTestCommandForBackend): RUN_afterCustomTestAction {
  return {
    type: ActionType.RUN_afterCustomTest,
    payload: delay(runAfterTest(runTestWithSolution))
  }
}

export function setShouldNotCountAsync(releaseCode: string): SET_shouldNotCountParticipationAction {
  return {
    type: ActionType.SET_shouldNotCountParticipation,
    payload: delay(setShouldNotCount(releaseCode))
  }
}


export function resetUserSolutionDoExerciseSiteAsync(releaseCode: string, pLangId: number): RESET_userSolutionAction {
  return {
    type: ActionType.RESET_userSolution,
    payload: delay(resetSolution(releaseCode, pLangId))
  }
}


/**
 * returns all possible custom user assets (files)
 */
export function getPossibleAllUserFilesAsync(): GET_allUserAssetPreviewsAction {
  return {
    type: ActionType.GET_allUserAssetPreviews,
    payload: delay(getPossibleAllUserFiles())
  }
}
