/**
 * Created by janis dähne.
 */
import {
  RESET_TestResultsAction,
  ResetAction,
  SET_addCustomProjectAssetFilesModalFilterTextAction,
  SET_addCustomProjectAssetFilesModalUploadPercentageAction,
  SET_addCustomProjectCustomTestAssetFilesModalFilterTextAction,
  SET_addCustomProjectCustomTestAssetFilesModalUploadPercentageAction,
  SET_botLeftPanelViewSizeAction,
  SET_botRightPanelViewSizeAction,
  SET_bottomLeftSelectedTabIndexAction,
  SET_bottomRightSelectedTabIndexAction,
  SET_displayNameAction,
  SET_hDividerPosOffsetAction,
  SET_initialPLangIdAction,
  SET_isAddCustomProjectAssetFilesModalDisplayedAction,
  SET_isAddCustomProjectCustomTestAssetFilesModalDisplayedAction,
  SET_isChangeTestDialogDisplayedAction,
  SET_isChoosePLangDialogDisplayedAction,
  SET_isCustomProjectCodeEditorSettingsDialogDisplayedAction,
  SET_isJustRunProgramArgsInputDisplayedAction,
  SET_justRunProgramArgsAction,
  SET_selectedConsoleViewTestAction,
  SET_topLeftPanelViewSizeAction,
  SET_topLeftSelectedTabIndexAction,
  SET_topRightPanelViewSizeAction,
  SET_topRightSelectedTabIndexAction,
  SET_vDividerPosOffsetAction
} from "../../reducers/editCustomProjectSite/editCustomProjectSiteReducer";
import {ActionType} from "../../reducers/editCustomProjectSite/editCustomProjectSiteActionTypes";
import {PanelViewSize} from "../../../components/panelSystem/panelGrid/panelGrid";
import {AwaitActions, MultiActions} from "../types";
import {loadManageCustomProjectSite} from "../manageCustomProjectsSite/manageCustomProjectsSiteActions";
import {getTestTypesAsync} from "../testTypesActions";
import {RunnableExerciseTestFrontendExclusive} from "../../../types/exerciseEditor";
import {RunnableCustomTestFrontendExclusive} from "../../../types/customTests";
import {AssessedSubmitTestFrontendExclusive} from "../../../types/testResults";
import {
  CustomProjectSolutionFileFullBase,
  RunnableCustomProjectTestFrontendExclusive
} from "../../../types/customProjects";
import {
  SET_mainFileIdAction,
  SET_solutionFilesAction
} from "../../reducers/editCustomProjectSite/subSets/setCustomProjectSolutionReducer";
import {setSolutionFiles_AndRefreshEditor} from "../../../helpers/editorHelper";
import {
  openTabChangeCustomProjectSolutionEditor,
  setSelectedTabIdCustomProjectSolutionEditor
} from "./editorActions/customProjectSolutionEditorActions";
import {getCustomProjectAsync, getCustomProjectTestResultsAsync} from "./editCustomProjectCrudActions";
import {setCustomProjectTests} from "./subSet/customProjectTestActions";
import {mergeCustomTestResultsCustomProject} from "../../../helpers/convertersAndTransformers";
import {getPLangsAsync} from '../plangSettingsView/pLangsCrudActions'


export function setCustomProjectDisplayName(displayName: string): SET_displayNameAction {
  return {
    type: ActionType.SET_displayName,
    displayName
  }
}

export function resetCustomProjectTestResults(): MultiActions {
  return (dispatch, getState) => {
    dispatch(setCustomProjectTests(mergeCustomTestResultsCustomProject(getState().editCustomProjectSiteState.customProject.tests, [], true)))
    dispatch(setSelectedConsoleViewTest(null))
  }
}

//-- normal solution
export function setSolutionFilesCustomProject(solutionFiles: ReadonlyArray<CustomProjectSolutionFileFullBase>): SET_solutionFilesAction {
  return {
    type: ActionType.SET_solutionFiles,
    solutionFiles
  }
}


export function onCreateCustomProjectSolutionFile(newFile: CustomProjectSolutionFileFullBase, allFiles: ReadonlyArray<CustomProjectSolutionFileFullBase>, selectedEditorTabId: number): AwaitActions {
  return async (dispatch, getState) => {

    const newFiles: ReadonlyArray<CustomProjectSolutionFileFullBase> = [...allFiles, newFile]

    await dispatch(setSolutionFiles_AndRefreshEditor(
      newFiles,
      setSolutionFilesCustomProject,
      )
    )

    if (allFiles.length === 0) {
      //this is the first file set it as the main file
      dispatch(setMainFileIdCustomProject(newFile.id))
    }

    //open the new file
    dispatch(openTabChangeCustomProjectSolutionEditor(newFile, newFiles, selectedEditorTabId))

  }
}

export function setMainFileIdCustomProject(mainFileId: number): SET_mainFileIdAction {
  return {
    type: ActionType.SET_mainFileId,
    mainFileId
  }
}

export function setCustomProjectCodeEditorSettingsDialogDisplayed(isDisplayed: boolean): SET_isCustomProjectCodeEditorSettingsDialogDisplayedAction {
  return {
    type: ActionType.SET_isCustomProjectCodeEditorSettingsDialogDisplayed,
    isDisplayed: isDisplayed
  }
}

export function setSelectedConsoleViewTest(selectedConsoleViewTest: RunnableExerciseTestFrontendExclusive
  | RunnableCustomTestFrontendExclusive | AssessedSubmitTestFrontendExclusive | RunnableCustomProjectTestFrontendExclusive | null): SET_selectedConsoleViewTestAction {
  return {
    type: ActionType.SET_selectedConsoleViewTest,
    selectedConsoleViewTest
  }
}

//--- panel indices

export function setTopRightSelectedTabIndex(index: number): SET_topRightSelectedTabIndexAction {
  return {
    type: ActionType.SET_topRightSelectedTabIndex,
    topRightSelectedTabIndex: index
  }
}

export function setTopLeftSelectedTabIndex(index: number): SET_topLeftSelectedTabIndexAction {
  return {
    type: ActionType.SET_topLeftSelectedTabIndex,
    topLeftSelectedTabIndex: index
  }
}

export function setBottomRightSelectedTabIndex(index: number): SET_bottomRightSelectedTabIndexAction {
  return {
    type: ActionType.SET_bottomRightSelectedTabIndex,
    bottomRightSelectedTabIndex: index
  }
}

export function setBottomLeftSelectedTabIndex(index: number): SET_bottomLeftSelectedTabIndexAction {
  return {
    type: ActionType.SET_bottomLeftSelectedTabIndex,
    bottomLeftSelectedTabIndex: index
  }
}


//--- dialog actions

export function setIsChangeTestDialogDisplayed(isDisplayed: boolean): SET_isChangeTestDialogDisplayedAction {
  return {
    type: ActionType.SET_isChangeTestDialogDisplayed,
    isDisplayed
  }
}


//--- panel view sizes

export function setTopRightPanelViewSize(topRightPanelViewSize: PanelViewSize): SET_topRightPanelViewSizeAction {
  return {
    type: ActionType.SET_topRightPanelViewSize,
    topRightPanelViewSize
  }
}

export function setTopLeftPanelViewSize(topLeftPanelViewSize: PanelViewSize): SET_topLeftPanelViewSizeAction {
  return {
    type: ActionType.SET_topLeftPanelViewSize,
    topLeftPanelViewSize
  }
}

export function setBotRightPanelViewSize(botRightPanelViewSize: PanelViewSize): SET_botRightPanelViewSizeAction {
  return {
    type: ActionType.SET_botRightPanelViewSize,
    botRightPanelViewSize
  }
}

export function setBotLeftPanelViewSize(botLeftPanelViewSize: PanelViewSize): SET_botLeftPanelViewSizeAction {
  return {
    type: ActionType.SET_botLeftPanelViewSize,
    botLeftPanelViewSize
  }
}


//--- console view settings

//--- panels grippers
export function setCustomProjectHDividerPosOffset(hDividerPosOffset: number): SET_hDividerPosOffsetAction {
  return {
    type: ActionType.SET_hDividerPosOffset,
    hDividerPosOffset
  }
}

export function setCustomProjectVDividerPosOffset(vDividerPosOffset: number): SET_vDividerPosOffsetAction {
  return {
    type: ActionType.SET_vDividerPosOffset,
    vDividerPosOffset
  }
}

export function setInitialPLangId(initialPLangId: number): SET_initialPLangIdAction {
  return {
    type: ActionType.SET_initialPLangId,
    initialPLangId
  }
}

//--- custom project description (markdown)

export function setIsAddCustomProjectAssetFilesModalDisplayed(isDisplayed: boolean, isBusy: boolean): SET_isAddCustomProjectAssetFilesModalDisplayedAction {
  return {
    type: ActionType.SET_isAddCustomProjectAssetFilesModalDisplayed,
    isDisplayed,
    isBusy
  }
}
export function setAddCustomProjectAssetFilesModalFilterText(addCustomProjectAssetFilesModalFilterText: string): SET_addCustomProjectAssetFilesModalFilterTextAction {
  return {
    type: ActionType.SET_addCustomProjectAssetFilesModalFilterText,
    addCustomProjectAssetFilesModalFilterText
  }
}

export function setAddCustomProjectAssetFilesModalUploadPercentage(addCustomProjectAssetFilesModalUploadPercentage: number | null | 'active'): SET_addCustomProjectAssetFilesModalUploadPercentageAction {
  return {
    type: ActionType.SET_addCustomProjectAssetFilesModalUploadPercentage,
    addCustomProjectAssetFilesModalUploadPercentage
  }
}

//--- custom project test asset files

export function setIsAddCustomProjectCustomTestAssetFilesModalDisplayed(isDisplayed: boolean, isBusy: boolean): SET_isAddCustomProjectCustomTestAssetFilesModalDisplayedAction {
  return {
    type: ActionType.SET_isAddCustomProjectCustomTestAssetFilesModalDisplayed,
    isDisplayed,
    isBusy
  }
}

export function setAddCustomProjectCustomTestAssetFilesModalFilterText(addCustomProjectCustomTestAssetFilesModalFilterText: string): SET_addCustomProjectCustomTestAssetFilesModalFilterTextAction {
  return {
    type: ActionType.SET_addCustomProjectCustomTestAssetFilesModalFilterText,
    addCustomProjectCustomTestAssetFilesModalFilterText
  }
}

export function setAddCustomProjectCustomTestAssetFilesModalUploadPercentage(addCustomProjectCustomTestAssetFilesModalUploadPercentage: number | null | 'active'): SET_addCustomProjectCustomTestAssetFilesModalUploadPercentageAction {
  return {
    type: ActionType.SET_addCustomProjectCustomTestAssetFilesModalUploadPercentage,
    addCustomProjectCustomTestAssetFilesModalUploadPercentage
  }
}

//---

export function resetCustomProjectTestResultsDuringSave(): MultiActions {
  return dispatch => {

    dispatch(_resetCustomProjectTestResultsDuringSave())
    dispatch(setSelectedConsoleViewTest(null))

  }
}

//resets test results
/**
 * call this during saving the solution
 * @returns {RESET_TestResultsAction}
 */
export function _resetCustomProjectTestResultsDuringSave(): RESET_TestResultsAction {
  return {
    type: ActionType.RESET_TestResults,
  }
}


export function loadEditCustomProjectSite(customProjectId: number | null, isCreating: boolean): AwaitActions {
  return async (dispatch, getState) => {

    dispatch(resetEditCustomProjectSite())

    //we only need to count to check if the user is allowed to create a new exercise (else disable button)
    //so we fetch all project previews
    //TODO normally the user should not get here if he has the max amount of projects (crate btn in overview should be disabled)... maybe remove??
    await dispatch(loadManageCustomProjectSite())

    await dispatch(getTestTypesAsync())

    await dispatch(getPLangsAsync())

    if (isCreating) {
      //nothing to do here
    } else {

      const lastEditedPLangId = getState().editCustomProjectSiteState.customProject.lastEditorPLangId

      await dispatch(getCustomProjectAsync(customProjectId))


      await dispatch(getCustomProjectTestResultsAsync(customProjectId, getState().editCustomProjectSiteState.customProject.solution.pLangId))

      // await dispatch(
      //   getDoExerciseAllTestResults(releaseCode, lastEditedPLangId, getState().doExerciseSiteState.solution.id))

      //try to select the first displayed tab (if any)
      if (getState().editCustomProjectSiteState.customProject.solution.solutionFiles.length > 0
        && getState().editCustomProjectSiteState.customProject.solution.solutionFiles.filter(
          p => p.uiIsDisplayed).length > 0) {
        dispatch(setSelectedTabIdCustomProjectSolutionEditor(
          getState().editCustomProjectSiteState.customProject.solution.solutionFiles.filter(
            p => p.uiIsDisplayed)[0].id))
      }


    }


  }
}

export function setJustRunProgramArgsCustomProject(justRunProgramArgs: string): SET_justRunProgramArgsAction {
  return {
    type: ActionType.SET_justRunProgramArgs,
    justRunProgramArgs
  }
}

export function setIsJustRunProgramArgsInputDisplayedCustomProject(isJustRunProgramArgsInputDisplayed: boolean): SET_isJustRunProgramArgsInputDisplayedAction {
  return {
    type: ActionType.SET_isJustRunProgramArgsInputDisplayed,
    isJustRunProgramArgsInputDisplayed
  }
}


export function setIsChoosePLangDialogDisplayed(isChoosePLangDialogDisplayed: boolean): SET_isChoosePLangDialogDisplayedAction {
  return {
    type: ActionType.SET_isChoosePLangDialogDisplayed,
    isChoosePLangDialogDisplayed
  }
}

export function resetEditCustomProjectSite(): ResetAction {
  return {
    type: ActionType.RESET
  }
}
