/**
 * Created by janis dähne.
 */
import {notExhaustive} from "../_notExhausiveHelper";
import {ActionBase, ActionType} from "./editorExerciseSiteActionTypes";
import {
  EditorExerciseForBackend,
  ExerciseCodeTemplateFullBase,
  FilePreviewFromBackend
} from "../../../types/exerciseEditor";
import {
  AllActions as GetEditorExerciseActionTypes,
  reducer as getEditorExerciseReducer
} from './getEditorExerciseReducer'

import {
  AllActions as CreateEditorExerciseActions,
  reducer as createEditorExerciseReducer
} from './createEditorExerciseReducer'
import {
  AllActions as UpdateEditorExerciseActions,
  reducer as updateEditorExerciseReducer
} from './updateEditorExerciseReducer'

import {
  AllActions as SetEditorExercisePropertiesActions,
  initial as initialExerciseProperties,
  reducer as setEditorExercisePropertiesReducer
} from './subSets/setEditorExercisePropertiesReducer'
import {
  AllActions as SetEditorExerciseDescriptionActions,
  initial as initialExerciseDescription,
  reducer as setEditorExerciseDescriptionReducer
} from './subSets/setEditorExerciseDescriptionReducer'

import {
  AllActions as SetEditorExerciseMetaDataActions,
  initial as initialExerciseMetaData,
  reducer as setEditorExerciseMetaDataReducer
} from './subSets/setEditorExerciseMetaDataReducer'
import {
  AllActions as SetEditorExerciseDefaultTestSettingsActions,
  initial as initialDefaultTestSettings,
  reducer as setEditorExerciseDefaultTestSettingsReducer
} from './subSets/setEditorExerciseDefaultTestSettingsReducer'

import {
  AllActions as SetEditorExerciseCodeTemplatesActions,
  initial as initialCodeTemplates,
  reducer as setEditorExerciseCodeTemplatesReducer
} from './subSets/setEditorExerciseCodeTemplatesReducer'
import {
  AllActions as SetEditorExerciseTestsActions,
  initial as initialTests,
  reducer as setEditorExerciseTestsReducer
} from './subSets/setEditorExerciseTestsReducer'

import {reducer as editorExerciseReducer, initial as initialEditorExercise} from './editorExerciseReducer'
import {UserGroupBase, UserGroupFromBackend, UserGroupWithExerciseCountFromBackend} from "../../../types/group";
import {TagFullBase} from "../../../types/tag";


import {ActionType as TagsActionsTypes} from "../manageTagsSite/manageTagsSiteActionTypes";
import {AllActions as GetTagActions, reducer as _getTagsReducer} from './_getTagsReducer'
import {ActionType as GroupExercisesSiteActionTypes} from "../manageGroupExercisesSite/groupExercisesSiteActionTypes";
import {AllActions as GetGroupsActions, reducer as _getGroupsReducer} from './_getGroupsReducer'
import {PanelViewSize} from "../../../components/panelSystem/panelGrid/panelGrid";
import {
  AllActions as AllLockExercisePermanentlyActions,
  reducer as lockExercisePermanentlyReducer
} from './crud/lockExercisePermanentlyReducer'

import {
  AllActions as AllAttachMarkdownAssetActions,
  reducer as attachMarkdownAssetReducer
} from './crud/assets/attachMarkdownAssetPreviewReducer'
import {
  AllActions as AllGetMarkdownAssetPreviewsActions,
  reducer as getMarkdownAssetPreviewsReducer
} from './crud/assets/getMarkdownAssetPreviewsReducer'
import {
  AllActions as AllDetachMarkdownAssetPreviewActions,
  reducer as detachMarkdownAssetPreviewReducer
} from './crud/assets/detachMarkdownAssetPreviewReducer'

import {
  AllActions as AllLoadSingleMarkdownAssetActions,
  reducer as loadSingleMarkdownAssetReducer
} from './crud/assets/loadSingleMarkdownAssetReducer'
import {
  AllActions as AllGetTestAssetPreviewsActions,
  reducer as getTestAssetPreviewsReducer
} from './crud/assets/getTestAssetPreviewsReducer'
import {
  AllActions as AllAttachTestAssetPreviewActions,
  reducer as attachTestAssetPreviewReducer
} from './crud/assets/attachTestAssetPreviewReducer'
import {
  AllActions as AllDetachTestAssetPreviewActions,
  reducer as detachTestAssetPreviewReducer
} from './crud/assets/detachTestAssetPreviewReducer'


//TODO move assets to own reducer because they are big and we don't want to copy them every time

export type UiCodeTemplateFilesDisplayedHash = {readonly [id: number]: boolean}

export type State = {

  readonly editorExercise: EditorExerciseForBackend
  /**
   * will be set to true after we created a new editor exercise successfully!
   * this is used so that we bypass the ask to leave dialog when we navigate to the edit exercise view (because now we got all the ids)
   * so this will be false when we loaded or create an exercise
   */
  readonly savedNewExercise: boolean

  /**
   * the old user group id when receiving an old exercise (or after updating) (to know which is the real user group even if the user changed to group in the frontend)
   * or -x/0 for invalid/not set e.g. new exercise
   */
  readonly oldGroupId: number

  /**
   * the val of IsOnlyVisibleToMe when the exercise was loaded/updated
   */
  readonly oldIsOnlyVisibleToMe: boolean
  /**
   * true: display the files in the templates panel view,
   * false: not
   */
  readonly uiCodeTemplateFilesDisplayedHash: UiCodeTemplateFilesDisplayedHash

  /**
   * the number of releases for this exercise
   */
  readonly numReleases: number


  readonly groups: ReadonlyArray<UserGroupFromBackend>
  readonly tags: ReadonlyArray<TagFullBase>

  readonly isLoading: boolean

  readonly topRightSelectedTabIndex: number
  readonly topLeftSelectedTabIndex: number
  readonly bottomRightSelectedTabIndex: number
  readonly bottomLeftSelectedTabIndex: number

  readonly topRightPanelViewSize: PanelViewSize
  readonly topLeftPanelViewSize: PanelViewSize
  readonly botRightPanelViewSize: PanelViewSize
  readonly botLeftPanelViewSize: PanelViewSize

  readonly hDividerPosOffset: number
  readonly vDividerPosOffset: number

  readonly isChangeTestDialogDisplayed: boolean
  readonly isChangeSelectedTagsDialogDisplayed: boolean
  readonly isChangeCodeTemplateDialogDisplayed: boolean
  readonly isChangeEditorSettingsDialogDisplayed: boolean

  //for the exercise description assets
  readonly isAddAssetFilesModalDisplayed: boolean
  readonly isAddAssetFilesModalBusy: boolean
  readonly addAssetFilesModalFilterText: string
  readonly addAssetFilesModalUploadPercentage: number | null | 'active'
  /**
   * all possible markdown asset file previews
   * set via CRUD
   */
  readonly allMarkdownAssetFilePreviews: ReadonlyArray<FilePreviewFromBackend>

  //for the test assets (we could place this also in the dialog reducer... but maybe we need this in the future in the wrapping (this) reducer)
  readonly isAddTestAssetFilesModalDisplayed: boolean
  readonly isAddTestAssetFilesModalBusy: boolean
  readonly addTestAssetFilesModalFilterText: string
  readonly addTestAssetFilesModalUploadPercentage: number | null | 'active'
  /**
   * all possible test asset file previews
   * set via CRUD
   */
  readonly allTestAssetFilePreviews: ReadonlyArray<FilePreviewFromBackend>




  /**
   * true: back button goes to own exercises site, false: group exercises site
   */
  readonly comesFromOwnExercisesSite: boolean
}


//function to evaluate getNextFrontendId every time
export const initial: () => State = () => {
  return {
    editorExercise: initialEditorExercise(),
    savedNewExercise: false,
    uiCodeTemplateFilesDisplayedHash: {},
    numReleases: 0, //allows editing code templates
    groups: [],
    tags: [],
    isLoading: false,
    oldGroupId: -1,
    oldIsOnlyVisibleToMe: false,

    topRightSelectedTabIndex: 0,
    topLeftSelectedTabIndex: 0,
    bottomRightSelectedTabIndex: 0,
    bottomLeftSelectedTabIndex: 0,

    topRightPanelViewSize: PanelViewSize.normal,
    topLeftPanelViewSize: PanelViewSize.normal,
    botRightPanelViewSize: PanelViewSize.normal,
    botLeftPanelViewSize: PanelViewSize.normal,

    isChangeTestDialogDisplayed: false,
    isChangeSelectedTagsDialogDisplayed: false,
    isChangeCodeTemplateDialogDisplayed: false,
    isChangeEditorSettingsDialogDisplayed: false,

    isAddAssetFilesModalDisplayed: false,
    isAddAssetFilesModalBusy: false,
    addAssetFilesModalFilterText: '',
    addAssetFilesModalUploadPercentage: null,
    allMarkdownAssetFilePreviews: [],


    isAddTestAssetFilesModalDisplayed: false,
    isAddTestAssetFilesModalBusy: false,
    addTestAssetFilesModalFilterText: '',
    addTestAssetFilesModalUploadPercentage: null,
    allTestAssetFilePreviews: [],


    hDividerPosOffset: 150,
    vDividerPosOffset: 150,

    comesFromOwnExercisesSite: true

  }
}

export interface SET_comesFromOwnExercisesSiteAction extends ActionBase {
  readonly type: ActionType.SET_comesFromOwnExercisesSite
  readonly comesFromOwnExercisesSite: boolean
}

export interface SET_editorExerciseAction extends ActionBase {
  readonly type: ActionType.SET_editorExercise
  readonly editorExercise: EditorExerciseForBackend
}

export interface SET_editorExerciseIdAction extends ActionBase {
  readonly type: ActionType.SET_editorExerciseId
  readonly id: number
}

export interface SET_noteAction extends ActionBase {
  readonly type: ActionType.SET_note
  readonly note: string
}

export interface SET_editorExerciseCreatorIdAction extends ActionBase {
  readonly type: ActionType.SET_editorExerciseCreatorId
  readonly creatorId: number
}

export interface SET_uiCodeTemplateFilesDisplayedHashAction extends ActionBase {
  readonly type: ActionType.SET_uiCodeTemplateFilesDisplayedHash
  readonly uiCodeTemplateFilesDisplayedHash: UiCodeTemplateFilesDisplayedHash
}


//--- panel selected indices

export interface SET_topRightSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_topRightSelectedTabIndex
  readonly topRightSelectedTabIndex: number
}

export interface SET_topLeftSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_topLeftSelectedTabIndex
  readonly topLeftSelectedTabIndex: number
}

export interface SET_bottomRightSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_bottomRightSelectedTabIndex
  readonly bottomRightSelectedTabIndex: number
}

export interface SET_bottomLeftSelectedTabIndexAction extends ActionBase {
  readonly type: ActionType.SET_bottomLeftSelectedTabIndex
  readonly bottomLeftSelectedTabIndex: number
}

// --- panel view sizes

export interface SET_topRightPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_topRightPanelViewSize
  readonly topRightPanelViewSize: PanelViewSize
}

export interface SET_topLeftPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_topLeftPanelViewSize
  readonly topLeftPanelViewSize: PanelViewSize
}

export interface SET_botRightPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_botRightPanelViewSize
  readonly botRightPanelViewSize: PanelViewSize
}

export interface SET_botLeftPanelViewSizeAction extends ActionBase {
  readonly type: ActionType.SET_botLeftPanelViewSize
  readonly botLeftPanelViewSize: PanelViewSize
}

//--- dialogs

export interface SET_isChangeTestDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isChangeTestDialogDisplayed
  readonly isDisplayed: boolean
}

export interface SET_isChangeSelectedTagsDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isChangeSelectedTagsDialogDisplayed
  readonly isDisplayed: boolean
}

export interface SET_isChangeCodeTemplateDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isChangeCodeTemplateDialogDisplayed
  readonly isDisplayed: boolean
}

export interface SET_isChangeEditorSettingsDialogDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isChangeEditorSettingsDialogDisplayed
  readonly isDisplayed: boolean
}

//--- exercise description / markdown asset files

export interface SET_isAddAssetFilesModalDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isAddAssetFilesModalDisplayed
  readonly isDisplayed: boolean
  readonly isBusy: boolean
}

export interface SET_addAssetFilesModalFilterTextAction extends ActionBase {
  readonly type: ActionType.SET_addAssetFilesModalFilterText
  readonly addAssetFilesModalFilterText: string
}

export interface SET_addAssetFilesModalUploadPercentageAction extends ActionBase {
  readonly type: ActionType.SET_addAssetFilesModalUploadPercentage
  readonly addAssetFilesModalUploadPercentage: number | null | 'active'
}

//--- exercise test asset files

export interface SET_isAddTestAssetFilesModalDisplayedAction extends ActionBase {
  readonly type: ActionType.SET_isAddTestAssetFilesModalDisplayed
  readonly isDisplayed: boolean
  readonly isBusy: boolean
}

export interface SET_addTestAssetFilesModalFilterTextAction extends ActionBase {
  readonly type: ActionType.SET_addTestAssetFilesModalFilterText
  readonly addTestAssetFilesModalFilterText: string
}

export interface SET_addTestAssetFilesModalUploadPercentageAction extends ActionBase {
  readonly type: ActionType.SET_addTestAssetFilesModalUploadPercentage
  readonly addTestAssetFilesModalUploadPercentage: number | null | 'active'
}


//--- panels

export interface SET_hDividerPosOffsetAction extends ActionBase {
  readonly type: ActionType.SET_hDividerPosOffset
  readonly hDividerPosOffset: number
}

export interface SET_vDividerPosOffsetAction extends ActionBase {
  readonly type: ActionType.SET_vDividerPosOffset
  readonly vDividerPosOffset: number
}

export interface ResetAction extends ActionBase {
  readonly type: ActionType.RESET
}

export type AllActions =
  ResetAction
  | SET_noteAction
  | SET_editorExerciseCreatorIdAction
  | SET_editorExerciseIdAction
  | SET_editorExerciseAction

  | SET_comesFromOwnExercisesSiteAction

  | SET_uiCodeTemplateFilesDisplayedHashAction
  | SET_hDividerPosOffsetAction
  | SET_vDividerPosOffsetAction

  | SET_topRightSelectedTabIndexAction
  | SET_topLeftSelectedTabIndexAction
  | SET_bottomRightSelectedTabIndexAction
  | SET_bottomLeftSelectedTabIndexAction

  | SET_topRightPanelViewSizeAction
  | SET_topLeftPanelViewSizeAction
  | SET_botRightPanelViewSizeAction
  | SET_botLeftPanelViewSizeAction

  | SET_isChangeSelectedTagsDialogDisplayedAction
  | SET_isChangeTestDialogDisplayedAction
  | SET_isChangeCodeTemplateDialogDisplayedAction
  | SET_isChangeEditorSettingsDialogDisplayedAction

  | SET_isAddAssetFilesModalDisplayedAction
  | SET_addAssetFilesModalFilterTextAction
  | SET_addAssetFilesModalUploadPercentageAction

  | SET_isAddTestAssetFilesModalDisplayedAction
  | SET_addTestAssetFilesModalFilterTextAction
  | SET_addTestAssetFilesModalUploadPercentageAction

  | GetEditorExerciseActionTypes
  | CreateEditorExerciseActions
  | UpdateEditorExerciseActions
  | AllLockExercisePermanentlyActions

  | SetEditorExercisePropertiesActions
  | SetEditorExerciseDescriptionActions
  | SetEditorExerciseMetaDataActions
  | SetEditorExerciseDefaultTestSettingsActions
  | SetEditorExerciseCodeTemplatesActions
  | SetEditorExerciseTestsActions
  | GetTagActions
  | GetGroupsActions

  | AllAttachMarkdownAssetActions
  | AllGetMarkdownAssetPreviewsActions
  | AllDetachMarkdownAssetPreviewActions
  | AllLoadSingleMarkdownAssetActions

  | AllGetTestAssetPreviewsActions
  | AllAttachTestAssetPreviewActions
  | AllDetachTestAssetPreviewActions


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_editorExercise:
      return {
        ...state,
        editorExercise: action.editorExercise
      }

    case TagsActionsTypes.GET_tags_PENDING:
    case TagsActionsTypes.GET_tags_FULFILLED:
    case TagsActionsTypes.GET_tags_REJECTED:
      return _getTagsReducer(state, action)

    case GroupExercisesSiteActionTypes.GET_groupsForExercises_PENDING:
    case GroupExercisesSiteActionTypes.GET_groupsForExercises_FULFILLED:
    case GroupExercisesSiteActionTypes.GET_groupsForExercises_REJECTED:
      return _getGroupsReducer(state, action)

    case ActionType.LOCK_exercisePermanently_PENDING:
    case ActionType.LOCK_exercisePermanently_FULFILLED:
    case ActionType.LOCK_exercisePermanently_REJECTED:
      return lockExercisePermanentlyReducer(state, action)

    case ActionType.GET_markdownAssetPreviews_PENDING:
    case ActionType.GET_markdownAssetPreviews_FULFILLED:
    case ActionType.GET_markdownAssetPreviews_REJECTED:
      return getMarkdownAssetPreviewsReducer(state, action)

    case ActionType.ATTACH_markdownAsset_PENDING:
    case ActionType.ATTACH_markdownAsset_FULFILLED:
    case ActionType.ATTACH_markdownAsset_REJECTED:
      return attachMarkdownAssetReducer(state, action)

    case ActionType.LOAD_markdownAsset_PENDING:
    case ActionType.LOAD_markdownAsset_FULFILLED:
    case ActionType.LOAD_markdownAsset_REJECTED:
      return loadSingleMarkdownAssetReducer(state, action)

    case ActionType.DETACH_markdownAsset_PENDING:
    case ActionType.DETACH_markdownAsset_FULFILLED:
    case ActionType.DETACH_markdownAsset_REJECTED:
      return detachMarkdownAssetPreviewReducer(state, action)


    case ActionType.GET_testAssetPreviews_PENDING:
    case ActionType.GET_testAssetPreviews_FULFILLED:
    case ActionType.GET_testAssetPreviews_REJECTED:
      return getTestAssetPreviewsReducer(state, action)

    case ActionType.ATTACH_testAsset_PENDING:
    case ActionType.ATTACH_testAsset_FULFILLED:
    case ActionType.ATTACH_testAsset_REJECTED:
      return attachTestAssetPreviewReducer(state, action)

    case ActionType.DETACH_testAsset_PENDING:
    case ActionType.DETACH_testAsset_FULFILLED:
    case ActionType.DETACH_testAsset_REJECTED:
      return detachTestAssetPreviewReducer(state, action)

    case ActionType.SET_editorExerciseId:
    case ActionType.SET_note:
    case ActionType.SET_editorExerciseCreatorId:
    case ActionType.SET_tests:
    case ActionType.SET_editorExerciseDefaultTestSettingsId:
    case ActionType.SET_timeoutInMs:
    case ActionType.SET_compileTimeoutInMs:
    case ActionType.SET_memoryLimitInKb:
    case ActionType.SET_maxDiskSpaceInKb:
    case ActionType.SET_editorExerciseMetaDataId:
    case ActionType.SET_tagIds:
    case ActionType.SET_editorExerciseDescriptionId:
    case ActionType.SET_editorExerciseContent:
    case ActionType.SET_assets:
    case ActionType.SET_displayName:
    case ActionType.SET_shortDescription:
    case ActionType.SET_isOnlyVisibleToMe:
    case ActionType.SET_canUserCreateCustomTests:
    case ActionType.SET_canUserCreateFiles:
    case ActionType.SET_showSubmitResults:
    case ActionType.SET_userGroupId:
      return {
        ...state,
        editorExercise: editorExerciseReducer(state.editorExercise, action)
      }

    case ActionType.SET_codeTemplates: {
      return {
        ...state,
        editorExercise: editorExerciseReducer(state.editorExercise, action),
        uiCodeTemplateFilesDisplayedHash: rebuildUiCodeTemplateFilesDisplayedHash(
          state.uiCodeTemplateFilesDisplayedHash, action.codeTemplates)
      }
    }

    case ActionType.GET_editorExercise_PENDING:
    case ActionType.GET_editorExercise_FULFILLED:
    case ActionType.GET_editorExercise_REJECTED:
      return getEditorExerciseReducer(state, action)

    case ActionType.CREATE_editorExercise_PENDING:
    case ActionType.CREATE_editorExercise_FULFILLED:
    case ActionType.CREATE_editorExercise_REJECTED:
      return createEditorExerciseReducer(state, action)


    case ActionType.UPDATE_editorExercise_PENDING:
    case ActionType.UPDATE_editorExercise_FULFILLED:
    case ActionType.UPDATE_editorExercise_REJECTED:
      return updateEditorExerciseReducer(state, action)


    case ActionType.SET_topRightSelectedTabIndex:
      return {
        ...state,
        topRightSelectedTabIndex: action.topRightSelectedTabIndex
      }

    case ActionType.SET_topLeftSelectedTabIndex:
      return {
        ...state,
        topLeftSelectedTabIndex: action.topLeftSelectedTabIndex
      }

    case ActionType.SET_bottomRightSelectedTabIndex:
      return {
        ...state,
        bottomRightSelectedTabIndex: action.bottomRightSelectedTabIndex
      }

    case ActionType.SET_bottomLeftSelectedTabIndex:
      return {
        ...state,
        bottomLeftSelectedTabIndex: action.bottomLeftSelectedTabIndex
      }

    case ActionType.SET_isChangeTestDialogDisplayed:
      return {
        ...state,
        isChangeTestDialogDisplayed: action.isDisplayed
      }

    case ActionType.SET_isChangeSelectedTagsDialogDisplayed:
      return {
        ...state,
        isChangeSelectedTagsDialogDisplayed: action.isDisplayed
      }
    case ActionType.SET_isChangeCodeTemplateDialogDisplayed:
      return {
        ...state,
        isChangeCodeTemplateDialogDisplayed: action.isDisplayed
      }

    case ActionType.SET_isChangeEditorSettingsDialogDisplayed:
      return {
        ...state,
        isChangeEditorSettingsDialogDisplayed: action.isDisplayed
      }

    case ActionType.SET_isAddAssetFilesModalDisplayed:
      return {
        ...state,
        isAddAssetFilesModalDisplayed: action.isDisplayed,
        isAddAssetFilesModalBusy: action.isBusy
      }
    case ActionType.SET_addAssetFilesModalFilterText:
      return {
        ...state,
        addAssetFilesModalFilterText: action.addAssetFilesModalFilterText
      }
    case ActionType.SET_addAssetFilesModalUploadPercentage:
      return {
        ...state,
        addAssetFilesModalUploadPercentage: action.addAssetFilesModalUploadPercentage
      }

    case ActionType.SET_isAddTestAssetFilesModalDisplayed:
      return {
        ...state,
        isAddTestAssetFilesModalDisplayed: action.isDisplayed,
        isAddTestAssetFilesModalBusy: action.isBusy
      }
    case ActionType.SET_addTestAssetFilesModalFilterText:
      return {
        ...state,
        addTestAssetFilesModalFilterText: action.addTestAssetFilesModalFilterText
      }
    case ActionType.SET_addTestAssetFilesModalUploadPercentage:
      return {
        ...state,
        addTestAssetFilesModalUploadPercentage: action.addTestAssetFilesModalUploadPercentage
      }

    case ActionType.SET_uiCodeTemplateFilesDisplayedHash:
      return {
        ...state,
        uiCodeTemplateFilesDisplayedHash: action.uiCodeTemplateFilesDisplayedHash
      }
    case ActionType.SET_hDividerPosOffset:
      return {
        ...state,
        hDividerPosOffset: action.hDividerPosOffset
      }
    case ActionType.SET_vDividerPosOffset:
      return {
        ...state,
        vDividerPosOffset: action.vDividerPosOffset
      }

    case ActionType.SET_topRightPanelViewSize:
      return {
        ...state,
        topRightPanelViewSize: action.topRightPanelViewSize
      }

    case ActionType.SET_topLeftPanelViewSize:
      return {
        ...state,
        topLeftPanelViewSize: action.topLeftPanelViewSize
      }

    case ActionType.SET_botRightPanelViewSize:
      return {
        ...state,
        botRightPanelViewSize: action.botRightPanelViewSize
      }

    case ActionType.SET_botLeftPanelViewSize:
      return {
        ...state,
        botLeftPanelViewSize: action.botLeftPanelViewSize
      }
    case ActionType.SET_comesFromOwnExercisesSite:
      return {
        ...state,
        comesFromOwnExercisesSite: action.comesFromOwnExercisesSite
      }

    case ActionType.RESET:
      return initial()

    default:
      notExhaustive(action)
      return state
  }
}

/**
 * rebuilds the UiCodeTemplateFilesDisplayedHash hash for the newCodeTemplates and tires to set the old visibility states
 * @param {UiCodeTemplateFilesDisplayedHash} oldUiCodeTemplateFilesDisplayedHash the old visibility values
 * @param {ReadonlyArray<ExerciseCodeTemplateFullBase>} newCodeTemplates the new code templates
 * @returns {UiCodeTemplateFilesDisplayedHash}
 */
function rebuildUiCodeTemplateFilesDisplayedHash(oldUiCodeTemplateFilesDisplayedHash: UiCodeTemplateFilesDisplayedHash,
                                                 newCodeTemplates: ReadonlyArray<ExerciseCodeTemplateFullBase>
): UiCodeTemplateFilesDisplayedHash {
  // noinspection TsLint
  let copy: UiCodeTemplateFilesDisplayedHash = {}

  newCodeTemplates.forEach(p => {
    copy = {
      ...copy,
      [p.id]: false
    }
  })

  for (const key in copy) {
    if (copy.hasOwnProperty(key)) {
      const oldVal = oldUiCodeTemplateFilesDisplayedHash[key]

      if (oldVal === true || oldVal === false) {
        copy = {
          ...copy,
          [key]: oldVal
        }
      }
    }
  }

  return copy

}
