/**
 * Created by janis dähne.
 */
import {Action} from "redux";
import {notExhaustive} from "../../_notExhausiveHelper";
import {TestSettingsFullBase} from "../../../../types/exerciseEditor";
import {ActionBase, ActionType} from "../editorExerciseSiteActionTypes";
import {getNextFrontendId} from "../../../../helpers/frontendIdHelper";

export type State = TestSettingsFullBase


export const initial: () => State = () => {
  return {
    id: getNextFrontendId(),
    timeoutInMs: 1000,
    memoryLimitInKb: 1000,
    maxDiskSpaceInKb: 1000,
    compileTimeoutInMs: 2000,
  }
}


export interface SET_editorExerciseDefaultTestSettingsIdAction extends ActionBase {
  readonly type: ActionType.SET_editorExerciseDefaultTestSettingsId
  readonly id: number
}

export interface SET_timeoutInMsAction extends ActionBase {
  readonly type: ActionType.SET_timeoutInMs
  readonly timeoutInMs: number
}

export interface SET_compileTimeoutInMsAction extends ActionBase {
  readonly type: ActionType.SET_compileTimeoutInMs
  readonly compileTimeoutInMs: number
}

export interface SET_memoryLimitInKbAction extends ActionBase {
  readonly type: ActionType.SET_memoryLimitInKb
  readonly memoryLimitInKb: number
}

export interface SET_maxDiskSpaceInKbAction extends ActionBase {
  readonly type: ActionType.SET_maxDiskSpaceInKb
  readonly maxDiskSpaceInKb: number
}


export type AllActions =
  SET_editorExerciseDefaultTestSettingsIdAction
  | SET_timeoutInMsAction
  | SET_compileTimeoutInMsAction
  | SET_memoryLimitInKbAction
  | SET_maxDiskSpaceInKbAction


export function reducer(state: State = initial(), action: AllActions): State {

  switch (action.type) {

    case ActionType.SET_editorExerciseDefaultTestSettingsId:
      return {
        ...state,
        id: action.id
      }

    case ActionType.SET_timeoutInMs:
      return {
        ...state,
        timeoutInMs: action.timeoutInMs
      }
    case ActionType.SET_compileTimeoutInMs:
      return {
        ...state,
        compileTimeoutInMs: action.compileTimeoutInMs
      }
    case ActionType.SET_memoryLimitInKb:
      return {
        ...state,
        memoryLimitInKb: action.memoryLimitInKb
      }
    case ActionType.SET_maxDiskSpaceInKb:
      return {
        ...state,
        maxDiskSpaceInKb: action.maxDiskSpaceInKb
      }


    default:
      notExhaustive(action)
      return state
  }
}

